## JSreturnFun2.R

## Version 2 of JSreturnFun.R  Updated March 2021.

## Likelihood and other functions for Jolly-Seber modified
## to model return times from temporary emigration.

## General functions:
## ------------------

## "log.fn" "logit.fn"  "expit.fn"  "lgp.fn"    "plg.fn"

## Functions for RET models:
## -------------------------

## Log likelihood function:

## "ret.like" 

## Wrapper functions for various models, all with return
## time tau modelled with a maximum M of non-breeding states. 
## Value is   mll = minus log likelihood:

## "cccr.mll"                Model beta(c) phi(c) p(c) tau(r)
## "tccr.mll"  "tccrNj.mll"  Model beta(t) phi(c) p(c) tau(r)
## "ctcr.mll"                Model beta(c) phi(t) p(c) tau(r)
## "ttcr.mll"  "ttcrNj.mll"  Model beta(t) phi(t) p(c) tau(r)
## "cctr.mll"                Model beta(c) phi(c) p(t) tau(r)
## "tctr.mll"  "tctrNj.mll"  Model beta(t) phi(c) p(t) tau(r)
## "cttr.mll"                Model beta(c) phi(t) p(t) tau(r)
## "tttr.mll"  "tttrNj.mll"  Model beta(t) phi(t) p(t) tau(r)

## where c = constant and t = time varying parameters.
## beta(c) really means one value for beta(0) then constant.
## Models with beta(t) come with two different parameterisations, 
## the basic one with superpopulation N and beta entry parameters,
## and a variant with current population Nj replacing N and the
## beta vector. This variant allows us to obtain standard errors
## for the Nj estimates directly from the hessian.
## To allow for parameter non-identifiability:
## Models with beta(t) and p(t) have p[1] = mean of later p[j] values.
## Models with phi(t) and p(t) have p[K] = mean of earlier p[j].


## Functions for Jolly-Seber models:

## "ccc.mll"               Model beta(c) phi(c) p(c)
## "tcc.mll"  "tccNj.mll"  Model beta(t) phi(c) p(c)
## "ctc.mll"               Model beta(c) phi(t) p(c)
## "ttc.mll"  "ttcNj.mll"  Model beta(t) phi(t) p(c)
## "cct.mll"               Model beta(c) phi(c) p(t)
## "tct.mll"  "tctNj.mll"  Model beta(t) phi(c) p(t)
## "ctt.mll"               Model beta(c) phi(t) p(t)
## "ttt.mll"  "tttNj.mll"  Model beta(t) phi(t) p(t)

## As before, beta(c) really means one value for beta(0) then constant.
## The Nj versions allow for direct estimation of SE(estimated Nj).
## To allow for parameter non-identifiability:
## Models with beta(t) and p(t) have p[1] = mean of later p[j] values.
## Models with phi(t) and p(t) have p[K] = mean of earlier p[j].



## ----------------------------------------------------------
##               General functions
## ----------------------------------------------------------

## Log function:
## ---------------

## Convert a vector to logs, don't allow to -Inf:

log.fn <- function(invec)
  {
  x <- invec
  x[x < 1e-20] <- 1e-20 
  y <- log(x)
  y
  }

## Logit function:
## ---------------

## Convert a vector to logits:

logit.fn <- function(invec)
  {
  y <- log.fn(invec/(1-invec))
  y
  }

## Expit function:
## ---------------

## Inverse of logit:

expit.fn <- function(invec)
   1/(1+exp(-invec))


## PLG function:
## -------------

## Convert probs (summing to 1) to logit function of conditional probs.
## (one fewer element). e.g. (a1,a2,a3,a4) goes to
## (logit(a1),logit(a2/(1-a1)),logit(a3/(1-a1-a2)))

plg.fn <- function(invect)
   {
   len <- length(invect)
   if (len==2) newvect <- invect[1]
   if (len>2)
      {
      # Cumulative sum:
      csin <- cumsum(invect)
      # Ratio vector:
      newvect <- invect[1:(len-1)]/(1-c(0,csin[1:(len-2)]))
      }
   # return logit:
   logit.fn(newvect)
   }


## LGP function:
## -------------

## Inverse of plg.fn.
## Logit ratios to probs (sum to 1) function, one more element.

lgp.fn <- function(invect)
   {
   len <- length(invect)
   if (len==1) p.v <- expit.fn(invect)
   if (len>1)
      {
      # Get ratios vector:
      ratvec <- expit.fn(invect)
      # Convert to beta vector: need cum product of 1-ratios
      cp <- cumprod(1-ratvec)
      # Return probability vector:
      p.v <- ratvec[1:len]*c(1,cp[1:(len-1)])
      }
   c(p.v,1-sum(p.v))
   }


## ----------------------------------------------------------
##                 RET functions
## ----------------------------------------------------------

## Likelihood:
## ------------

## Input scalar N, vector beta (length K), vector phi (length K-1), 
## vector p (length K), vector tau (length M+1).
## Sum(beta)=1, sum(tau)=1.

ret.like <- function(N.s,beta.v,phi.v,p.v,tau.v){
  loglike <- -1000000  # Safeguard against going out of bounds
  if ((min(c(N.max-N.s,N.s-n,tau.v,p.v,phi.v,beta.v))>=0)&
      (max(c(tau.v,p.v,phi.v,beta.v))<=1)){
    # Lengthen tau vector to K-1:
    tau.v <- c(tau.v,rep(0,K-1-length(tau.v)))
    # Set up gamma.m, matrix of probs for Type S segments,
    # and alpha.m, matrix of probs for Type A segments.
    # gamma[i,j] = prob(FBIS at i and first seen at j).
    # alpha[i,j] = prob(next seen at j given seen at i), j>i.
    gamma.m <- diag(beta.v*p.v)   # Seen when FBIS
    alpha.m <- matrix(NA,K,K)
    for (ii in 1:(K-1)) for (jj in (ii+1):K){
      # Simple case first, seen next year:
      if (jj == ii + 1){
        gamma.m[ii,jj] <- beta.v[ii]*(1-p.v[ii])*phi.v[ii]*tau.v[1]*p.v[jj]
        alpha.m[ii,jj] <- phi.v[ii]*tau.v[1]*p.v[jj]
      }
      # Other cases:
      if (jj > ii + 1){
        # Set up all combos of B/N between i and j:
        AA <- jj - ii
        sumterms <- 0
        combos.m <- as.matrix(expand.grid( rep(list(0:1),AA-1))[,1:(AA-1)])
        for (cc in 1:nrow(combos.m)){
          b.v <- c(combos.m[cc,],1)  # Vector to indicate breeding years
          k.v <- (1:AA)[b.v==1]        # List of breeding years, ending AA
          ytb.v <- diff(c(0,k.v))      # Vector of years to next breeding
          # Tau product for inner term
          term1 <- prod(tau.v[ytb.v])
          # (1-p) product for calendar years of internal B:
          term2 <- prod((1-p.v)[k.v[-length(k.v)]+ii])
          # Full a[ij], adjusted for being caught at AA:
          sumterms <- sumterms + term1*term2
        }
        gamma.m[ii,jj] <- beta.v[ii]*(1-p.v[ii])*
          prod(phi.v[ii:(jj-1)])*sumterms*p.v[jj]
        alpha.m[ii,jj] <- prod(phi.v[ii:(jj-1)])*sumterms*p.v[jj]
      }
    }
    gamma.m[is.na(gamma.m)] <- 0
    # Set up gamma.v, vector of probs. first seen at j:
    gamma.v <- apply(gamma.m,2,sum)
    # Construct alpha probs vector for use in likelihood:
    alpha.v <- c(alpha.m[!is.na(alpha.m)])
    # Find chi.v, probabilities for Type E segments:
    chi.v <- rep(1,K)
    for (jj in 1:(K-1)) chi.v[jj] <- 1 - sum(alpha.m[jj,(jj+1):K])
    # Nu, overall proportion not captured.
    nu <- 1 - sum(gamma.v)
    # Construct multinomial likelihood for these segments:
    # Now, omitting constant terms sum(lgamma(ns.vect+1)),
    # sum(lgamma(na.vect+1)) and sum(lgamma(ne.vect+1)),
    u.v <- u.vect
    u.v[u.v==0] <- 1   # Need to allow for possible u=0
    loglike <- lgamma(N.s + 1) - lgamma(N.s - n + 1) -
               sum(lgamma(u.v)) +
               sum(ns.vect[gamma.v>0]*log.fn(gamma.v[gamma.v>0])) +
               sum(na.vect[alpha.v>0]*log.fn(alpha.v[alpha.v>0])) +
               sum(ne.vect[chi.v>0]*log.fn(chi.v[chi.v>0]))
    if (nu>0) loglike <- loglike + (N.s - n)*log.fn(nu)
  }
  loglike
}


## -------------------------------------------------------
##       Wrapper functions for different return time models
## -------------------------------------------------------


## -------------------------------------------------------
##             beta(c) phi(c) p(c) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi[1]),
## logit(p[1]), plg(tau).

cccr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- rep(expit.fn(invect[3]),K-1)
  p.in     <- rep(expit.fn(invect[4]),K)
  tau.in   <- lgp.fn(invect[(5):(M+4)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}


## -------------------------------------------------------
##             beta(t) phi(c) p(c) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi[1]),
## logit(p[1]), plg(tau).

tccr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  p.in     <- rep(expit.fn(invect[K+2]),K)
  tau.in   <- lgp.fn(invect[(K+3):(K+M+2)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

tccrNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- rep(expit.fn(invect[K+2]),K)
  tau.in   <- lgp.fn(invect[(K+3):(K+M+2)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}


## -------------------------------------------------------
##             beta(c) phi(t) p(c) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi),
## logit(p[1]), plg(tau).

ctcr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- expit.fn(invect[3:(K+1)])
  p.in     <- rep(expit.fn(invect[K+2]),K)
  tau.in   <- lgp.fn(invect[(K+3):(K+M+2)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}


## -------------------------------------------------------
##             beta(t) phi(t) p(c) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi),
## logit(p[1]), plg(tau).

ttcr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  p.in     <- rep(expit.fn(invect[2*K]),K)
  tau.in   <- lgp.fn(invect[(2*K+1):(2*K+M)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

ttcrNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- rep(expit.fn(invect[2*K]),K)
  tau.in   <- lgp.fn(invect[(2*K+1):(2*K+M)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

## -------------------------------------------------------
##             beta(c) phi(c) p(t) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi[1]),
## logit(p), plg(tau).

cctr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- rep(expit.fn(invect[3]),K-1)
  p.in     <- expit.fn(invect[4:(K+3)])
  tau.in   <- lgp.fn(invect[(K+4):(K+M+3)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}


## -------------------------------------------------------
##             beta(t) phi(c) p(t) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi[1]),
## logit(p[-1]), plg(tau).
## Note: p[1] is estimated as mean of later p[j].

tctr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(mean(p.in),p.in)
  tau.in   <- lgp.fn(invect[(2*K+1):(2*K+M)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

tctrNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(mean(p.in),p.in)
  tau.in   <- lgp.fn(invect[(2*K+1):(2*K+M)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

## -------------------------------------------------------
##             beta(c) phi(t) p(t) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi),
## logit(p[-K]), plg(tau).
## Note: p[K] is estimated as mean of earlier p[j].

cttr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- expit.fn(invect[3:(K+1)])
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(p.in,mean(p.in))
  tau.in   <- lgp.fn(invect[(2*K+1):(2*K+M)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}


## -------------------------------------------------------
##             beta(t) phi(t) p(t) tau(r)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi),
## logit(p[-c(1,K)]), plg(tau).
## Note: p[1] and p[K] are set to mean p[2:(K-1)]

tttr.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  p.in     <- expit.fn(invect[(2*K):(3*K-3)])
  p.in     <- c(mean(p.in),p.in,mean(p.in))
  tau.in   <- lgp.fn(invect[(3*K-2):(3*K+M-3)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

tttrNj.mll <- function(invect){
  Nj.in     <- exp(invect[1:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- expit.fn(invect[(2*K):(3*K-3)])
  p.in     <- c(mean(p.in),p.in,mean(p.in))
  tau.in   <- lgp.fn(invect[(3*K-2):(3*K+M-3)])
  mll <- -ret.like(N.in,beta.in,phi.in,p.in,tau.in)
}

## ----------------------------------------------------------
##                 JSSA functions
## ----------------------------------------------------------

## Modified above functions, return every year,
## tau becomes just the constant 1.
## FBIS is now first recruited (first time present).
## Combos not needed, as breeding every year.

## Likelihood:
## ------------

## Input N, beta, phi (length K-1), p (length K).
## Already have p1 set to mean(p), sum(beta)=1.

jssa.like <- function(N.s,beta.v,phi.v,p.v){
  loglike <- -1000000
  if ((min(c(N.max-N.s,N.s-n,p.v,phi.v,beta.v))>=0)&
      (max(c(p.v,phi.v,beta.v))<=1)){
    # Set up gamma.m, matrix of probs for Type S segments,
    # and alpha.m, matrix of probs for Type A segments.
    # gamma[i,j] = prob(FBIS at i and first seen at j).
    # alpha[i,j] = prob(next seen at j given seen at i), j>i.
    gamma.m <- diag(beta.v*p.v)   # Seen when FBIS
    alpha.m <- matrix(NA,K,K)
    for (ii in 1:(K-1)) for (jj in (ii+1):K){
      # Simple case first, seen next year:
      if (jj == ii + 1){
        gamma.m[ii,jj] <- beta.v[ii]*(1-p.v[ii])*phi.v[ii]*p.v[jj]
        alpha.m[ii,jj] <- phi.v[ii]*p.v[jj]
      }
      # Other cases:
      if (jj > ii + 1){
        # Survival, non-captures between ii and jj, then capture at jj:
        alpha.m[ii,jj] <-
          prod(phi.v[ii:(jj-1)])*prod(1-p.v[(ii+1):(jj-1)])*p.v[jj]
        gamma.m[ii,jj] <- beta.v[ii]*(1-p.v[ii])*alpha.m[ii,jj]
      }
    }
    gamma.m[is.na(gamma.m)] <- 0
    gamma.m.save <<- gamma.m
    # Set up gamma.v, vector of probs. first seen at j:
    gamma.v <- apply(gamma.m,2,sum)
    gamma.save <<- gamma.v
    alpha.m.save <<- alpha.m
    # Construct alpha probs vector for use in likelihood:
    alpha.v <- c(alpha.m[!is.na(alpha.m)])
    alpha.v.save <<- alpha.v
    N.save <<- N.s
    # Find chi.v, probabilities for Type E segments:
    chi.v <- rep(1,K)
    for (jj in 1:(K-1)) chi.v[jj] <- 1 - sum(alpha.m[jj,(jj+1):K])
    chi.save <<- chi.v
    # Nu, overall proportion not captured.
    nu <- 1 - sum(gamma.v)
    nu.save <<- nu
    # Construct multinomial likelihood for these segments:
    loglike <- lgamma(N.s + 1) - lgamma(N.s - n + 1) -
               sum(lgamma(u.vect)) +
               sum(ns.vect[gamma.v>0]*log.fn(gamma.v[gamma.v>0])) +
               sum(na.vect[alpha.v>0]*log.fn(alpha.v[alpha.v>0])) +
               sum(ne.vect[chi.v>0]*log.fn(chi.v[chi.v>0]))
    if (nu>0) loglike <- loglike + (N.s - n)*log.fn(nu)
  }
  loglike
}


## -------------------------------------------------------
##       Wrapper functions for different JSSA models
## -------------------------------------------------------

## -------------------------------------------------------
##             beta(c) phi(c) p(c)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi[1]),
## logit(p[1]).

ccc.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- rep(expit.fn(invect[3]),K-1)
  p.in     <- rep(expit.fn(invect[4]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}


## -------------------------------------------------------
##             beta(t) phi(c) p(c)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi[1]),
## logit(p[1]).

tcc.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  p.in     <- rep(expit.fn(invect[K+2]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

tccNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- rep(expit.fn(invect[K+2]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

## -------------------------------------------------------
##             beta(c) phi(t) p(c)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi),
## logit(p[1]).

ctc.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- expit.fn(invect[3:(K+1)])
  p.in     <- rep(expit.fn(invect[K+2]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}


## -------------------------------------------------------
##             beta(t) phi(t) p(c)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi),
## logit(p[1]).

ttc.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  p.in     <- rep(expit.fn(invect[2*K]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

ttcNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- rep(expit.fn(invect[2*K]),K)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

## -------------------------------------------------------
##             beta(c) phi(c) p(t)
## -------------------------------------------------------

## Input vector is log.fn(N), logit(beta[1]), logit(phi[1]),
## logit(p).

cct.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- rep(expit.fn(invect[3]),K-1)
  p.in     <- expit.fn(invect[4:(K+3)])
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}


## -------------------------------------------------------
##             beta(t) phi(c) p(t)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi[1]),
## logit(p).
## Note: p[1] is estimated as mean of later p[j].

tct.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(mean(p.in),p.in)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

tctNj.mll <- function(invect){
  Nj.in    <- exp(invect[1:K])
  phi.in   <- rep(expit.fn(invect[K+1]),K-1)
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(mean(p.in),p.in)
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

## -------------------------------------------------------
##             beta(c) phi(t) p(t)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta[1:2]), logit(phi),
## logit(p).
## Note: p[K] is estimated as mean of earlier p[j].

ctt.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- expit.fn(invect[2])
  beta.in  <- c(beta.in,rep((1-beta.in)/(K-1),K-1))
  phi.in   <- expit.fn(invect[3:(K+1)])
  p.in     <- expit.fn(invect[(K+2):(2*K)])
  p.in     <- c(p.in,mean(p.in))
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}


## -------------------------------------------------------
##             beta(t) phi(t) p(t)
## -------------------------------------------------------

## Input vector is log.fn(N), plg(beta), logit(phi),
## logit(p).
## Note: p[1] and p[K] are estimated as mean of p[2:(K-1)].

ttt.mll <- function(invect){
  N.in     <- exp(invect[1])
  beta.in  <- lgp.fn(invect[2:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  p.in     <- expit.fn(invect[(2*K):(3*K-3)])
  p.in     <- c(mean(p.in),p.in,mean(p.in))
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

tttNj.mll <- function(invect){
  Nj.in     <- exp(invect[1:K])
  phi.in   <- expit.fn(invect[(K+1):(2*K-1)])
  B.in     <- Nj.in
  B.in[-1] <- B.in[-1] - Nj.in[-K]*phi.in[-K]
  ### Add new line, stop B<0:
  B.in[B.in<0.0001] <- 0.0001
  N.in     <- sum(B.in)
  beta.in  <- B.in/N.in
  p.in     <- expit.fn(invect[(2*K):(3*K-3)])
  p.in     <- c(mean(p.in),p.in,mean(p.in))
  mll <- -jssa.like(N.in,beta.in,phi.in,p.in)
}

## -------------------------------------------------------
## -------------------------------------------------------

