## JSreturnRun2.R   

## Likelihood and other functions for Jolly-Seber modified
## to model return times from temporary emigration.

## Analysis of capture matrix data set.
## Run eight return-time models and their Jolly-Seber-type counterparts.

## Copy and paste the commands from this file into R.

## Note: Some models, especially those with time-dependent survival
## probabilities, phi(t), may give a warning that not all standard errors
## are estimable. The warning is:
##    "In sqrt(diag(vcovmat)) : NaNs produced"
## All the point estimates are valid, as are the confidence intervals
## which are given in the output. See the file "readme2.pdf" for details.

########################################################################
##       Preliminaries, setting up the data and constants
########################################################################

## Empty the workspace:
## --------------------

rm(list=ls())


## Read in the functions:
## ----------------------

source("JSreturnFun2.R")     ## The file needs to be in this directory


## Read in the data set:
## ---------------------

x.mat <- as.matrix(read.csv("testdataK10.csv",T))

## Check that all elements are either 0 or 1:
table(x.mat)

## Check there are no rows with all zeros:
table(apply(x.mat,1,sum))

## Note: It is possible to have a column of zeros, if one year's sampling had
## to be missed. Include that column in the data, so that the annual survival
## estimates don't lose track of the years.

## Set up list of results:
## -----------------------

results.list <- list(cccr=NA,tccr=NA,ctcr=NA,ttcr=NA,  ## Return time models
                     cctr=NA,tctr=NA,cttr=NA,tttr=NA,
		     ccc=NA,tcc=NA,ctc=NA,ttc=NA,      ## Jolly-Seber models
                     cct=NA,tct=NA,ctt=NA,ttt=NA)
		   

## Set some constants:
## -------------------

M <- 2    ## M = presumed maximum no. of non-breeding years before return
          ## See the file "readme2.pdf" for details on selecting M.

K <- ncol(x.mat)   ## K = number of samples (years)

n <- nrow(x.mat)   ## n = no. of individuals

n.vect <- apply(x.mat,2,sum)   ## Numbers caught per sample

N.max  <- 5*n      ## Set a high maximum for N (superpopulation size).


## Minimal sufficient statistics needed for return analysis
## --------------------------------------------------------

## There are three segment types:
## Type S (start) segment:  capture history (CH) up to first capture, e.g. 001.
## Type A (adult, alive) segment:  e.g.(1)00001  follows a one, ends with a one.
## Type E (end) segment:  (1)000    follows a one, all zeros to end of study.

## The segment types must be tabulated:

## ns.vect = vector of numbers of S-type segments.
## na.mat  = matrix of numbers of A-type segments a[i,j].
## ne.vect = vector of numbers of E-type segments.

## Construct ns.vect, na.mat and ne.vect:

ns.vect <- rep(0,K)
na.mat  <- matrix(0,K,K)
ne.vect <- rep(0,K)

for (i in 1:n)
   {
   ones <- (1:K)[x.mat[i,]==1]  # List of occasions when seen
   ns.vect[ones[1]] <- ns.vect[ones[1]] + 1
   n.ones <- length(ones)    # Number of ones
   last.one <- ones[n.ones]
   # Number last caught at j:
   ne.vect[last.one] <- ne.vect[last.one] + 1
   # Type A segments:
   if (n.ones>1) for (j in 1:(n.ones-1))
      {
      occs <- ones[j:(j+1)]  # Two successive occasions
      na.mat[occs[1],occs[2]] <-  na.mat[occs[1],occs[2]] + 1
      }
   }
for (i in 1:K) for (j in 1:K) if (i>=j) na.mat[i,j] <- NA

## Set up other objects:

na.vect <- c(na.mat[!is.na(na.mat)])   # vector used in likelihood
u.vect  <- ns.vect       ## Numbers first caught at samples 1 to K

## No. bits info:

nbits <- 2*K + K*(K-1)/2

## Calculate observed taus, assuming p=1. 
## To be used as start for optimisation.

tau.obs <- rep(0,K-1)
for (ii in 1:(K-1)) for (jj in (ii+1):K)
  tau.obs[jj-ii] <- tau.obs[jj-ii] + na.mat[ii,jj]
tau.obs <- tau.obs/sum(tau.obs)

plot(1:(K-1),tau.obs,
     xlab="Time units until first observed return",
     ylab="Proportion",
     main="Observed taus")
lines(1:(K-1),tau.obs,lty=5)



########################################################################
##             Fit Return Time Models
########################################################################

## Set starting values:
## --------------------

N.start    <- n*1.5
beta.start <- rep(1/K,K)
phi.start  <- 0.8
p.start    <- 0.5
tau.start  <- tau.obs[1:(M+1)]
tau.start  <- tau.start/sum(tau.start)


## --------------------------------------------------------
##              cccr = beta(c).phi(c).p(c).tau(r)
## --------------------------------------------------------

## Initialise:

start.vect <- c(log.fn(N.start),logit.fn(beta.start[1]),
                logit.fn(c(phi.start,p.start)),
                plg.fn(tau.start))

print(-cccr.mll(start.vect))

## Fit model:

print(date())
cccr.fit <- optim(start.vect,cccr.mll,
                  method="L-BFGS-B",hessian=T)
print(-cccr.fit$value)
print(cccr.fit$con)
print(date())   # Dates are shown to help monitor time taken

## Optional extra starts if not converged:

for (aa in 1:3) if (cccr.fit$con!=0){
  print(date())
  start.vect <- cccr.fit$par
  print(aa)
  cccr.fit <- optim(start.vect,cccr.mll,
             method="L-BFGS-B",hessian=T)
  print(-cccr.fit$value)
  print(cccr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -cccr.fit$value
rd       <- -2*max.ll
npar     <- length(cccr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- cccr.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3])
p.out    <- expit.fn(outvect[4])
tau.out  <- lgp.fn(outvect[(5):(M+4)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(cccr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[3]
  selogitphi <- ses[3]
  upperphi <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi <- expit.fn(logitphi - 1.96*selogitphi)
  logitp   <- outvect[4]
  selogitp <- ses[4]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
cccr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(cccr.out)
results.list$cccr <- cccr.out



## Set this as new starting point for the next three models:

N.start        <- cccr.out$N[1]
names(N.start) <- NULL
beta.start     <- cccr.out$beta
phi.start      <- cccr.out$phi[1]
names(phi.start) <- NULL
p.start        <- cccr.out$p[1]
names(p.start) <- NULL
tau.start      <- cccr.out$tau
Nj.start       <- cccr.out$Nj

## --------------------------------------------------------
##              tccr = beta(t).phi(c).p(c).tau(r)
## --------------------------------------------------------

## Initialise from cccr model:

start.vect <- c(log.fn(cccr.out$N[1]),
                plg.fn(cccr.out$beta),
                logit.fn(c(cccr.out$phi[1],cccr.out$p[[1]])),
                plg.fn(cccr.out$tau))
names(start.vect) <- NULL

print(-tccr.mll(start.vect))

## Fit model:

print(date())
tccr.fit <- optim(start.vect,tccr.mll,
                 method="L-BFGS-B",hessian=T)
print(-tccr.fit$value)
print(tccr.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tccr.fit$con!=0){
  print(date())
  start.vect <- tccr.fit$par
  print(aa)
  tccr.fit <- optim(start.vect,tccr.mll,
             method="L-BFGS-B",hessian=T)
  print(-tccr.fit$value)
  print(tccr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tccr.fit$value
rd       <- -2*max.ll
npar     <- length(tccr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- tccr.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[K+1])
p.out    <- expit.fn(outvect[K+2])
tau.out  <- lgp.fn(outvect[(K+3):(K+M+2)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tccr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[K+1]
  selogitphi <- ses[K+1]
  upperphi <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi <- expit.fn(logitphi - 1.96*selogitphi)
  logitp   <- outvect[K+2]
  selogitp <- ses[K+2]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
tccr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(tccr.out)

results.list$tccr <- tccr.out


## Variant with Nj parameters:
## ---------------------------

## Initialise:

Nj.start <- tccr.out$Nj

start.vect <- c(log.fn(Nj.start),
                logit.fn(c(phi.start,p.start)),
                plg.fn(tau.start))

print(-tccrNj.mll(start.vect))

## Fit model:

print(date())
tccrNj.fit <- optim(start.vect,tccrNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tccrNj.fit$value)
print(tccrNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tccrNj.fit$con!=0){
  print(date())
  start.vect <- tccrNj.fit$par
  print(aa)
  tccrNj.fit <- optim(start.vect,tccrNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tccrNj.fit$value)
  print(tccrNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tccrNj.fit$value
rd       <- -2*max.ll
npar     <- length(tccrNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tccrNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tccrNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tccrNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tccrNj.out)


## --------------------------------------------------------
##              ctcr = beta(c).phi(t).p(c).tau(r)
## --------------------------------------------------------

## Initialise from cccr model:

start.vect <- c(log.fn(cccr.out$N[1]),
                logit.fn(cccr.out$beta[1]),
                rep(logit.fn(cccr.out$phi[1]),K-1),
                logit.fn(cccr.out$p[[1]]),
                plg.fn(cccr.out$tau))
names(start.vect) <- NULL

print(-ctcr.mll(start.vect))

## Fit model:

print(date())
ctcr.fit <- optim(start.vect,ctcr.mll,
                 method="L-BFGS-B",hessian=T)
print(-ctcr.fit$value)
print(ctcr.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (ctcr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ctcr.fit$par
  ctcr.fit <- optim(start.vect,ctcr.mll,
             method="L-BFGS-B",hessian=T)
  print(-ctcr.fit$value)
  print(ctcr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ctcr.fit$value
rd       <- -2*max.ll
npar     <- length(ctcr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ctcr.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3:(K+1)])
p.out    <- expit.fn(outvect[K+2])
tau.out  <- lgp.fn(outvect[(K+3):(K+M+2)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ctcr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitp   <- outvect[K+2]
  selogitp <- ses[K+2]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
ctcr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(ctcr.out)

results.list$ctcr <- ctcr.out

## --------------------------------------------------------
##              ttcr = beta(t).phi(t).p(c).tau(r)
## --------------------------------------------------------

## Initialise from either ctcr or tccr, whichever had higher LL:

start.ctcr <- c(log.fn(ctcr.out$N[1]),
                plg.fn(ctcr.out$beta),
                logit.fn(c(ctcr.out$phi,ctcr.out$p[1])),
                plg.fn(ctcr.out$tau))
names(start.ctcr) <- NULL
print(-ttcr.mll(start.ctcr))

start.tccr <- c(log.fn(tccr.out$N[1]),
                plg.fn(tccr.out$beta),
                rep(logit.fn(tccr.out$phi[1]),K-1),
                logit.fn(tccr.out$p[1]),
                plg.fn(tccr.out$tau))
names(start.tccr) <- NULL
print(-ttcr.mll(start.tccr))

start.vect <- start.ctcr
if (-ttcr.mll(start.tccr) > -ttcr.mll(start.ctcr))
    start.vect <- start.tccr

## Fit model:

print(date())
ttcr.fit <- optim(start.vect,ttcr.mll,
                 method="L-BFGS-B",hessian=T)
print(-ttcr.fit$value)
print(ttcr.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (ttcr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ttcr.fit$par
  ttcr.fit <- optim(start.vect,ttcr.mll,
             method="L-BFGS-B",hessian=T)
  print(-ttcr.fit$value)
  print(ttcr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ttcr.fit$value
rd       <- -2*max.ll
npar     <- length(ttcr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ttcr.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[(K+1):(2*K-1)])
p.out    <- expit.fn(outvect[2*K])
tau.out  <- lgp.fn(outvect[(2*K+1):(2*K+M)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ttcr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitp   <- outvect[2*K]
  selogitp <- ses[2*K]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
ttcr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(ttcr.out)

results.list$ttcr <- ttcr.out

## Variant with Nj parameters:
## ---------------------------

## Initialise from  model above:

start.vect <- c(log.fn(ttcr.out$Nj),
                logit.fn(c(ttcr.out$phi,ttcr.out$p[1])),
                plg.fn(ttcr.out$tau))
names(start.vect) <- NULL

print(-ttcrNj.mll(start.vect))

## Fit model:

print(date())
ttcrNj.fit <- optim(start.vect,ttcrNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-ttcrNj.fit$value)
print(ttcrNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (ttcrNj.fit$con!=0){
  print(date())
  start.vect <- ttcrNj.fit$par
  print(aa)
  ttcrNj.fit <- optim(start.vect,ttcrNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-ttcrNj.fit$value)
  print(ttcrNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ttcrNj.fit$value
rd       <- -2*max.ll
npar     <- length(ttcrNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- ttcrNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ttcrNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))

## -------------
## Save results:
## -------------
ttcrNj.out <- list(info=out1, Nj=Nj.mat)

print(ttcrNj.out)



## --------------------------------------------------------
##              cctr = beta(c).phi(c).p(t).tau(r)
## --------------------------------------------------------

## Initialise from cccr model:

start.vect <- c(log.fn(cccr.out$N[1]),
                logit.fn(cccr.out$beta[1]),
                logit.fn(cccr.out$phi[1]),
                rep(logit.fn(cccr.out$p[1]),K),
                plg.fn(cccr.out$tau))
names(start.vect) <- NULL

print(-cctr.mll(start.vect))

## Fit model:

print(date())
cctr.fit <- optim(start.vect,cctr.mll,
                 method="L-BFGS-B",hessian=T)
print(-cctr.fit$value)
print(cctr.fit$con)
print(date())   


## Optional extra starts if not converged:

for (aa in 1:3) if (cctr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- cctr.fit$par
  cctr.fit <- optim(start.vect,cctr.mll,
             method="L-BFGS-B",hessian=T)
  print(-cctr.fit$value)
  print(cctr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -cctr.fit$value
rd       <- -2*max.ll
npar     <- length(cctr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- cctr.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3])
p.out    <- expit.fn(outvect[4:(K+3)])
tau.out  <- lgp.fn(outvect[(K+4):(K+M+3)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(cctr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[3]
  selogitphi <- ses[3]
  upperphi   <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi   <- expit.fn(logitphi - 1.96*selogitphi)
}
## -------------
## Save results:
## -------------
cctr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=p.out,
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(cctr.out)

results.list$cctr <- cctr.out

## --------------------------------------------------------
##              tctr = beta(t).phi(c).p(t).tau(r)
## --------------------------------------------------------

## Initialise from either cctr or tccr, whichever had higher LL:

start.cctr <- c(log.fn(cctr.out$N[1]),
                plg.fn(cctr.out$beta),
                logit.fn(cctr.out$phi[1]),
                logit.fn(cctr.out$p[-1]),
                plg.fn(cctr.out$tau))
names(start.cctr) <- NULL
print(-tctr.mll(start.cctr))

start.tccr <- c(log.fn(tccr.out$N[1]),
                plg.fn(tccr.out$beta),
                logit.fn(tccr.out$phi[1]),
                rep(logit.fn(tccr.out$p[1]),K-1),
                plg.fn(tccr.out$tau))
names(start.tccr) <- NULL
print(-tctr.mll(start.tccr))

start.vect <- start.cctr
if (-tctr.mll(start.tccr) > -tctr.mll(start.cctr))
    start.vect <- start.tccr
    
## Fit model:

print(date())
tctr.fit <- optim(start.vect,tctr.mll,
                   method="L-BFGS-B",hessian=T,
                   control=list(maxit=30))
print(-tctr.fit$value)
print(tctr.fit$con)
print(date()) 

## Optional extra starts if not converged:

for (aa in 1:3) if (tctr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tctr.fit$par
  tctr.fit <- optim(start.vect,tctr.mll,
             method="L-BFGS-B",hessian=T,
                   control=list(maxit=30))
  print(-tctr.fit$value)
  print(tctr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tctr.fit$value
rd       <- -2*max.ll
npar     <- length(tctr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- tctr.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[K+1])
p.out    <- c(NA,expit.fn(outvect[(K+2):(2*K)]))
tau.out  <- lgp.fn(outvect[(2*K+1):(2*K+M)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tctr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[K+1]
  selogitphi <- ses[K+1]
  upperphi   <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi   <- expit.fn(logitphi - 1.96*selogitphi)
}
## -------------
## Save results:
## -------------
tctr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=p.out,
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(tctr.out)

results.list$tctr <- tctr.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from previous model:

start.vect <- c(log.fn(tctr.out$Nj),
                logit.fn(c(tctr.out$phi[1],tctr.out$p[-1])),
                plg.fn(tctr.out$tau))

print(-tctrNj.mll(start.vect))

## Fit model:

print(date())
tctrNj.fit <- optim(start.vect,tctrNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tctrNj.fit$value)
print(tctrNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tctrNj.fit$con!=0){
  print(date())
  start.vect <- tctrNj.fit$par
  print(aa)
  tctrNj.fit <- optim(start.vect,tctrNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tctrNj.fit$value)
  print(tctrNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tctrNj.fit$value
rd       <- -2*max.ll
npar     <- length(tctrNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tctrNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tctrNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tctrNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tctrNj.out)


## --------------------------------------------------------
##              cttr = beta(c).phi(t).p(t).tau(r)
## --------------------------------------------------------

## Initialise from either cctr or ctcr, whichever has higher LL:

start.cctr <- c(log.fn(cctr.out$N[1]),
                logit.fn(cctr.out$beta[1]),
                rep(logit.fn(cctr.out$phi[1]),K-1),
                logit.fn(cctr.out$p[-K]),
                plg.fn(cctr.out$tau))
names(start.cctr) <- NULL
print(-cttr.mll(start.cctr))

start.ctcr <- c(log.fn(ctcr.out$N[1]),
                logit.fn(ctcr.out$beta[1]),
                logit.fn(ctcr.out$phi),
                rep(logit.fn(ctcr.out$p[1]),K-1),
                plg.fn(ctcr.out$tau))
names(start.ctcr) <- NULL
print(-cttr.mll(start.ctcr))

start.vect <- start.cctr
if (-cttr.mll(start.ctcr) > -cttr.mll(start.cctr))
    start.vect <- start.ctcr

## Fit model:

print(date())
cttr.fit <- optim(start.vect,cttr.mll,
                  method="L-BFGS-B",hessian=T,
                  control=list(maxit=30))
print(-cttr.fit$value)
print(cttr.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (cttr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- cttr.fit$par
  cttr.fit <- optim(start.vect,cttr.mll,
                    method="L-BFGS-B",hessian=T,
                    control=list(maxit=30))
  print(-cttr.fit$value)
  print(cttr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -cttr.fit$value
rd       <- -2*max.ll
npar     <- length(cttr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- cttr.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3:(K+1)])
p.out    <- c(expit.fn(outvect[(K+2):(2*K)]),NA)
tau.out  <- lgp.fn(outvect[(2*K+1):(2*K+M)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tctr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
}
## -------------
## Save results:
## -------------
cttr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=p.out,
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(cttr.out)

results.list$cttr <- cttr.out

## --------------------------------------------------------
##              tttr = beta(t).phi(t).p(t).tau(r)
## --------------------------------------------------------

## Initialise from cttr, tctr or ttcr, whichever has highest LL:

start.cttr <- c(log.fn(cttr.out$N[1]),
                plg.fn(cttr.out$beta),
                logit.fn(c(cttr.out$phi,cttr.out$p[-c(1,K)])),
                plg.fn(cttr.out$tau))
names(start.cttr) <- NULL
print(-tttr.mll(start.cttr))

start.tctr <- c(log.fn(tctr.out$N[1]),
                plg.fn(tctr.out$beta),
                rep(logit.fn(tctr.out$phi[1]),K-1),
                logit.fn(tctr.out$p[-c(1,K)]),
                plg.fn(tctr.out$tau))
names(start.tctr) <- NULL
print(-tttr.mll(start.tctr))

start.ttcr <- c(log.fn(ttcr.out$N[1]),
                plg.fn(ttcr.out$beta),
                logit.fn(ttcr.out$phi),
                rep(logit.fn(ttcr.out$p[1]),K-2),
                plg.fn(ttcr.out$tau))
names(start.ttcr) <- NULL
print(-tttr.mll(start.ttcr))

start.vect <- start.cttr
if (-tttr.mll(start.tctr) > -tttr.mll(start.cttr))
    start.vect <- start.tctr
if (-tttr.mll(start.ttcr) > -tttr.mll(start.vect))
    start.vect <- start.ttcr
print(-tttr.mll(start.vect))


## Fit model:

print(date())
tttr.fit <- optim(start.vect,tttr.mll,
                 method="L-BFGS-B",hessian=T)
print(-tttr.fit$value)
print(tttr.fit$con)
print(date()) 


## Optional extra starts if not converged:

for (aa in 1:3) if (tttr.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tttr.fit$par
  tttr.fit <- optim(start.vect,tttr.mll,
             method="L-BFGS-B",hessian=T)
  print(-tttr.fit$value)
  print(tttr.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tttr.fit$value
rd       <- -2*max.ll
npar     <- length(tttr.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- tttr.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[(K+1):(2*K-1)])
p.out    <- c(NA,expit.fn(outvect[(2*K):(3*K-3)]),NA)
tau.out  <- lgp.fn(outvect[(3*K-2):(3*K+M-3)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tttr.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
}
## -------------
## Save results:
## -------------
tttr.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=p.out,
                  tau=tau.out,
                  Births=B.out,
                  Nj=Nj.out)

print(tttr.out)

results.list$tttr <- tttr.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from  model above:

start.vect <- c(log.fn(tttr.out$Nj),
                logit.fn(c(tttr.out$phi,tttr.out$p[-c(1,K)])),
                plg.fn(tttr.out$tau))
names(start.vect) <- NULL

print(-tttrNj.mll(start.vect))

## Fit model:

print(date())
tttrNj.fit <- optim(start.vect,tttrNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tttrNj.fit$value)
print(tttrNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tttrNj.fit$con!=0){
  print(date())
  start.vect <- tttrNj.fit$par
  print(aa)
  tttrNj.fit <- optim(start.vect,tttrNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tttrNj.fit$value)
  print(tttrNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tttrNj.fit$value
rd       <- -2*max.ll
npar     <- length(tttrNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tttrNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tttrNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tttrNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tttrNj.out)



## --------------------------------------------------------
##              JSSA model fitting
## --------------------------------------------------------

## --------------------------------------------------------
##              ccc = beta(c).phi(c).p(c)
## --------------------------------------------------------

## Initialise:

start.vect <- c(log.fn(N.start),logit.fn(beta.start[1]),
                logit.fn(c(phi.start,p.start)))
names(start.vect) <- NULL
print(-ccc.mll(start.vect))

## Fit model:

print(date())
ccc.fit <- optim(start.vect,ccc.mll,
                 method="L-BFGS-B",hessian=T)
print(-ccc.fit$value)
print(ccc.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (ccc.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ccc.fit$par
  ccc.fit <- optim(start.vect,ccc.mll,
             method="L-BFGS-B",hessian=T)
  print(-ccc.fit$value)
  print(ccc.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ccc.fit$value
rd       <- -2*max.ll
npar     <- length(ccc.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ccc.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3])
p.out    <- expit.fn(outvect[4])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ccc.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[3]
  selogitphi <- ses[3]
  upperphi <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi <- expit.fn(logitphi - 1.96*selogitphi)
  logitp   <- outvect[4]
  selogitp <- ses[4]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
ccc.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  Births=B.out,
                  Nj=Nj.out)

print(ccc.out)

results.list$ccc <- ccc.out

## --------------------------------------------------------
##              tcc = beta(t).phi(c).p(c)
## --------------------------------------------------------

## Initialise from previous model:

start.vect <- c(log.fn(ccc.out$N[1]),plg.fn(ccc.out$beta),
                logit.fn(c(ccc.out$phi[1],ccc.out$p[1])))
names(start.vect) <- NULL

print(-tcc.mll(start.vect))

## Fit model:

print(date())
tcc.fit <- optim(start.vect,tcc.mll,
                 method="L-BFGS-B",hessian=T)
print(-tcc.fit$value)
print(tcc.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (tcc.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tcc.fit$par
  tcc.fit <- optim(start.vect,tcc.mll,
             method="L-BFGS-B",hessian=T)
  print(-tcc.fit$value)
  print(tcc.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tcc.fit$value
rd       <- -2*max.ll
npar     <- length(tcc.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- tcc.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[K+1])
p.out    <- expit.fn(outvect[K+2])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tcc.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[K+1]
  selogitphi <- ses[K+1]
  upperphi <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi <- expit.fn(logitphi - 1.96*selogitphi)
  logitp   <- outvect[K+2]
  selogitp <- ses[K+2]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
tcc.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  Births=B.out,
                  Nj=Nj.out)

print(tcc.out)

results.list$tcc <- tcc.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from previous model:

start.vect <- c(log.fn(tcc.out$Nj),
                logit.fn(c(tcc.out$phi[1],tcc.out$p[1])))
names(start.vect) <- NULL

print(-tccNj.mll(start.vect))

## Fit model:

print(date())
tccNj.fit <- optim(start.vect,tccNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tccNj.fit$value)
print(tccNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tccNj.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tccNj.fit$par
  tccNj.fit <- optim(start.vect,tccNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tccNj.fit$value)
  print(tccNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tccNj.fit$value
rd       <- -2*max.ll
npar     <- length(tccNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tccNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tccNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tccNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tccNj.out)

## --------------------------------------------------------
##              ctc = beta(c).phi(t).p(c)
## --------------------------------------------------------

## Initialise from ccc model:

start.vect <- c(log.fn(ccc.out$N[1]),logit.fn(ccc.out$beta[1]),
                rep(logit.fn(ccc.out$phi[1]),K-1),
                logit.fn(ccc.out$p[1]))
names(start.vect) <- NULL

print(-ctc.mll(start.vect))

## Fit model:

print(date())
ctc.fit <- optim(start.vect,ctc.mll,
                 method="L-BFGS-B",hessian=T)
print(-ctc.fit$value)
print(ctc.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (ctc.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ctc.fit$par
  ctc.fit <- optim(start.vect,ctc.mll,
             method="L-BFGS-B",hessian=T)
  print(-ctc.fit$value)
  print(ctc.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ctc.fit$value
rd       <- -2*max.ll
npar     <- length(ctc.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ctc.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3:(K+1)])
p.out    <- expit.fn(outvect[K+2])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ctc.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitp   <- outvect[K+2]
  selogitp <- ses[K+2]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
ctc.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  Births=B.out,
                  Nj=Nj.out)

print(ctc.out)

results.list$ctc <- ctc.out


## --------------------------------------------------------
##              ttc = beta(t).phi(t).p(c)
## --------------------------------------------------------

## Initialise from tcc model:

start.vect <- c(log.fn(tcc.out$N[1]),plg.fn(tcc.out$beta),
                rep(logit.fn(tcc.out$phi[1]),K-1),
                logit.fn(tcc.out$p[1]))
names(start.vect) <- NULL

print(-ttc.mll(start.vect))

## Fit model:

print(date())
ttc.fit <- optim(start.vect,ttc.mll,
                 method="L-BFGS-B",hessian=T)
print(-ttc.fit$value)
print(ttc.fit$con)
print(date())

## Optional extra starts if not converged:

for (aa in 1:3) if (ttc.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ttc.fit$par
  ttc.fit <- optim(start.vect,ttc.mll,
             method="L-BFGS-B",hessian=T)
  print(-ttc.fit$value)
  print(ttc.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ttc.fit$value
rd       <- -2*max.ll
npar     <- length(ttc.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ttc.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[(K+1):(2*K-1)])
p.out    <- expit.fn(outvect[2*K])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperp <- NA
lowerp <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ttc.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitp   <- outvect[2*K]
  selogitp <- ses[2*K]
  upperp   <- expit.fn(logitp + 1.96*selogitp)
  lowerp   <- expit.fn(logitp - 1.96*selogitp)
}
## -------------
## Save results:
## -------------
ttc.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=c(p.hat=p.out,p.lower=lowerp,
                    p.upper=upperp),
                  Births=B.out,
                  Nj=Nj.out)

print(ttc.out)

results.list$ttc <- ttc.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from previous model:

start.vect <- c(log.fn(ttc.out$Nj),
                logit.fn(c(ttc.out$phi,ttc.out$p[1])))
names(start.vect) <- NULL

print(-ttcNj.mll(start.vect))

## Fit model:

print(date())
ttcNj.fit <- optim(start.vect,ttcNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-ttcNj.fit$value)
print(ttcNj.fit$con)
print(date())     

## Optional extra starts if not converged:

for (aa in 1:3) if (tccNj.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tccNj.fit$par
  tccNj.fit <- optim(start.vect,tccNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tccNj.fit$value)
  print(tccNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ttcNj.fit$value
rd       <- -2*max.ll
npar     <- length(ttcNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- ttcNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ttcNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
ttcNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(ttcNj.out)

## --------------------------------------------------------
##              cct = beta(c).phi(c).p(t)
## --------------------------------------------------------

## Initialise from ccc:

start.vect <- c(log.fn(ccc.out$N[1]),
                logit.fn(ccc.out$beta[1]),
                logit.fn(ccc.out$phi[1]),
                rep(logit.fn(ccc.out$p[1]),K))
names(start.vect) <- NULL

print(-cct.mll(start.vect))

## Fit model:

print(date())
cct.fit <- optim(start.vect,cct.mll,
                 method="L-BFGS-B",hessian=T)
print(-cct.fit$value)
print(cct.fit$con)
print(date())


## Optional extra starts if not converged:

for (aa in 1:3) if (cct.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- cct.fit$par
  cct.fit <- optim(start.vect,cct.mll,
             method="L-BFGS-B",hessian=T)
  print(-cct.fit$value)
  print(cct.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -cct.fit$value
rd       <- -2*max.ll
npar     <- length(cct.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- cct.fit$par
names(outvect) <- NULL
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3])
p.out    <- expit.fn(outvect[4:(K+3)])
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(cct.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[3]
  selogitphi <- ses[3]
  upperphi   <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi   <- expit.fn(logitphi - 1.96*selogitphi)
}
## -------------
## Save results:
## -------------
cct.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=p.out,
                  Births=B.out,
                  Nj=Nj.out)

print(cct.out)

results.list$cct <- cct.out


## --------------------------------------------------------
##              tct = beta(t).phi(c).p(t)
## --------------------------------------------------------

## Initialise from tcc:

start.vect <- c(log.fn(tcc.out$N[1]),
                plg.fn(tcc.out$beta),
                logit.fn(tcc.out$phi[1]),
                rep(logit.fn(tcc.out$p[1]),K-1))
names(start.vect) <- NULL

print(-tct.mll(start.vect))

## Fit model:

print(date())
tct.fit <- optim(start.vect,tct.mll,
                 method="L-BFGS-B",hessian=T)
print(-tct.fit$value)
print(tct.fit$con)
print(date())


## Optional extra starts if not converged:

for (aa in 1:3) if (tct.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tct.fit$par
  tct.fit <- optim(start.vect,tct.mll,
             method="L-BFGS-B",hessian=T)
  print(-tct.fit$value)
  print(tct.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tct.fit$value
rd       <- -2*max.ll
npar     <- length(tct.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- tct.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[K+1])
p.out    <- c(NA,expit.fn(outvect[(K+2):(2*K)]))
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
upperphi <- NA
lowerphi <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tct.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
  logitphi   <- outvect[K+1]
  selogitphi <- ses[K+1]
  upperphi   <- expit.fn(logitphi + 1.96*selogitphi)
  lowerphi   <- expit.fn(logitphi - 1.96*selogitphi)
}
## -------------
## Save results:
## -------------
tct.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=c(phi.hat=phi.out,phi.lower=lowerphi,
                    phi.upper=upperphi),
                  p=p.out,
                  Births=B.out,
                  Nj=Nj.out)

print(tct.out)

results.list$tct <- tct.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from previous model:

start.vect <- c(log.fn(tct.out$Nj),
                logit.fn(c(tct.out$phi[1],tct.out$p[-1])))
names(start.vect) <- NULL

print(-tctNj.mll(start.vect))

## Fit model:

print(date())
tctNj.fit <- optim(start.vect,tctNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tctNj.fit$value)
print(tctNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tccNj.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tccNj.fit$par
  tccNj.fit <- optim(start.vect,tccNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tccNj.fit$value)
  print(tccNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tctNj.fit$value
rd       <- -2*max.ll
npar     <- length(tctNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tctNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tctNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tctNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tctNj.out)



## --------------------------------------------------------
##              ctt = beta(c).phi(t).p(t)
## --------------------------------------------------------

## Initialise from cct:

start.vect <- c(log.fn(cct.out$N[1]),
                logit.fn(cct.out$beta[1]),
                rep(logit.fn(cct.out$phi[1]),K-1),
                logit.fn(cct.out$p[-K]))
names(start.vect) <- NULL

print(-ctt.mll(start.vect))

## Fit model:

print(date())
ctt.fit <- optim(start.vect,ctt.mll,
                 method="L-BFGS-B",hessian=T)
print(-ctt.fit$value)
print(ctt.fit$con)
print(date())


## Optional extra starts if not converged:

for (aa in 1:3) if (ctt.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ctt.fit$par
  ctt.fit <- optim(start.vect,ctt.mll,
             method="L-BFGS-B",hessian=T)
  print(-ctt.fit$value)
  print(ctt.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ctt.fit$value
rd       <- -2*max.ll
npar     <- length(ctt.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ctt.fit$par
N.out    <- exp(outvect[1])
beta.out <- expit.fn(outvect[2])
beta.out <- c(beta.out,rep((1-beta.out)/(K-1),K-1))
phi.out  <- expit.fn(outvect[3:(K+1)])
p.out    <- c(expit.fn(outvect[(K+2):(2*K)]),NA)
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ctt.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
}
## -------------
## Save results:
## -------------
ctt.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=p.out,
                  Births=B.out,
                  Nj=Nj.out)

print(ctt.out)

results.list$ctt <- ctt.out


## --------------------------------------------------------
##              ttt = beta(t).phi(t).p(t)
## --------------------------------------------------------

## Initialise from tct:

start.vect <- c(log.fn(tct.out$N[1]),
                plg.fn(tct.out$beta),
                rep(logit.fn(tct.out$phi[1]),K-1),
                logit.fn(tct.out$p[-c(1,K)]))
names(start.vect) <- NULL

print(-ttt.mll(start.vect))

## Fit model:

print(date())
ttt.fit <- optim(start.vect,ttt.mll,
                 method="L-BFGS-B",hessian=T)
print(-ttt.fit$value)
print(ttt.fit$con)
print(date()) 


## Optional extra starts if not converged:

for (aa in 1:3) if (ttt.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- ttt.fit$par
  ttt.fit <- optim(start.vect,ttt.mll,
             method="L-BFGS-B",hessian=T)
  print(-ttt.fit$value)
  print(ttt.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -ttt.fit$value
rd       <- -2*max.ll
npar     <- length(ttt.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect  <- ttt.fit$par
N.out    <- exp(outvect[1])
beta.out <- lgp.fn(outvect[2:K])
phi.out  <- expit.fn(outvect[(K+1):(2*K-1)])
p.out    <- c(NA,expit.fn(outvect[(2*K):(3*K-3)]),NA)
B.out    <- N.out*beta.out
Nj.out   <- B.out
for (j in 2:K) 
  Nj.out[j] <- B.out[j] + Nj.out[j-1]*phi.out[j-1]
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperN <- NA
lowerN <- NA
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(ttt.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logN   <- outvect[1]
  selogN <- ses[1]
  upperN <- exp(logN + 1.96*selogN)
  lowerN <- exp(logN - 1.96*selogN)
}
## -------------
## Save results:
## -------------
ttt.out <- list(info=out1,
                  N=c(N.hat=N.out,N.lower=lowerN,N.upper=upperN),
                  beta=beta.out,
                  phi=phi.out,
                  p=p.out,
                  Births=B.out,
                  Nj=Nj.out)

print(ttt.out)

results.list$ttt <- ttt.out


## Variant with Nj parameters:
## ---------------------------

## Initialise from previous model:

start.vect <- c(log.fn(ttt.out$Nj),
                logit.fn(c(ttt.out$phi,ttt.out$p[-c(1,K)])))
names(start.vect) <- NULL

print(-tttNj.mll(start.vect))

## Fit model:

print(date())
tttNj.fit <- optim(start.vect,tttNj.mll,
                 method="L-BFGS-B",hessian=T)
print(-tttNj.fit$value)
print(tttNj.fit$con)
print(date())   

## Optional extra starts if not converged:

for (aa in 1:3) if (tttNj.fit$con!=0){
  print(date())
  print(aa)
  start.vect <- tttNj.fit$par
  tttNj.fit <- optim(start.vect,tttNj.mll,
             method="L-BFGS-B",hessian=T)
  print(-tttNj.fit$value)
  print(tttNj.fit$con)
  print(date())
}

## Construct output:

## -----
## Info:
## -----
max.ll   <- -tttNj.fit$value
rd       <- -2*max.ll
npar     <- length(tttNj.fit$par)
AIC      <- rd + 2*npar
AICc     <- AIC + 2*npar*(npar+1)/(nbits-npar-1)
BIC      <- rd + npar*log.fn(nbits)
out1     <- c(Max.LL=max.ll,ResDev=rd,npar=npar,
              AIC=AIC,AICc=AICc,BIC=BIC)
## ---------
## Par ests:
## ---------
outvect   <- tttNj.fit$par
Nj.out    <- exp(outvect[1:K])
## ---------------------
## Confidence intervals:
## ---------------------
ses <- rep(NA,npar)
upperNj <- rep(NA,K)
lowerNj <- rep(NA,K)
if (exists("vcovmat")) rm(vcovmat)
vcovmat <- try(solve(tttNj.fit$hessian),silent=T)
if (is.matrix(vcovmat)){
  ses  <- sqrt(diag(vcovmat))
  logNj   <- outvect[1:K]
  selogNj <- ses[1:K]
  upperNj <- exp(logNj + 1.96*selogNj)
  lowerNj <- exp(logNj - 1.96*selogNj)
}
Nj.mat <- cbind(Nj.out,lowerNj,upperNj)
dimnames(Nj.mat) <- list(NULL,c("Nj.estimate","Lower CL","Upper CL"))
## -------------
## Save results:
## -------------
tttNj.out <- list(info=out1,
                  Nj=Nj.mat)

print(tttNj.out)


###################################################################
##             Build information criteria table
###################################################################

model.list <- results.list
nmods <- length(model.list)
IC.mat <- matrix(NA,nmods,8)
dimnames(IC.mat) <- list(names(model.list),
                         c("LogLike","Npar","AIC","AICc","BIC",
                           "relAIC","relAICc","relBIC"))
for (i in 1:nmods){
  this.mod <-  model.list[[i]]
  IC.mat[i,1:5] <- this.mod[[1]][c(1,3:6)]
}
IC.mat[,6] <- round(IC.mat[,3] - min(IC.mat[,3]),1)
IC.mat[,7] <- round(IC.mat[,4] - min(IC.mat[,4]),1)
IC.mat[,8] <- round(IC.mat[,5] - min(IC.mat[,5]),1)

print(IC.mat)

###################################################################
##             Compare models graphically
###################################################################

## Plot LL versus npar:

plot(IC.mat[,c(2,1)], pch = rep(1:8,2), cex = 1.5,
     col=c(rep("red",8),rep("blue",8)),
     main = "LL versus npar, 16 models, K10 data")
legend(5,-760,c("ccc","tcc","ctc","cct","ttc","tct","ctt","ttt"),
       pch=1:8)
legend(10,-780,c("Return-time models","Jolly-Seber models"),
       pch = rep(16,2), col = c("red","blue"))

## Plot N estimates:

Nests.mat <- matrix(NA,nmods,3)
dimnames(Nests.mat) <- list(names(model.list),
                            c("N.hat","lower95%","upper95%"))
for (i in 1:nmods){
  this.mod <-  model.list[[i]]
  Nests.mat[i,1:3] <- this.mod$N
}

plot(1:16,Nests.mat[,1], type = 'n',
     ylim = range(Nests.mat),
     xlab = "Models", ylab = "N estimate",
     main = "95% confidence intervals for N",
     axes = F)
axis(1,c(4.5,12.5),c("Return Models","JS Models"))
axis(2)
box()
for (i in 1:8) lines(x = c(i,i),
    y = c(Nests.mat[i,2], Nests.mat[i,3]), col="red")
for (i in 9:16) lines(x = c(i,i),
    y = c(Nests.mat[i,2], Nests.mat[i,3]), col="blue")
points(1:16, Nests.mat[,2], pch=16,
       col = c(rep("red",8), rep("blue",8)))
points(1:16, Nests.mat[,3], pch=16,
       col = c(rep("red",8), rep("blue",8)))
points(1:16, Nests.mat[,1], pch = rep(1:8,2),
    cex = 1.5,col = c(rep("red",8),rep("blue",8)))
legend(1,625,c("cccr","tccr","ctcr","cctr","ttcr","tctr","cttr","tttr"),
       pch=1:8, col="red")
legend(6,625,c("ccc","tcc","ctc","cct","ttc","tct","ctt","ttt"),
       pch=1:8, col  = "blue")
    





## Other options:

## Try sequences of models with M varying, choose among them using AIC or BIC.


## Note about parameters in Pledger, Baker and Scribner (2013):

## gamma.vect = vector of unconditional probabilities for first seen at j.
## alpha.mat  = upper triangular matrix of probabilities of A-type segments,
## alpha[i,j] = prob(next seen at j, given seen at i)
## chi.vect   = vector of probabilities of Type E segments,
## chi[k]     = prob(not seen after k, given seen at k),
## tau.vect   = vector of estimated return times.

